//
//  RKTableController.m
//  RestKit
//
//  Created by Blake Watters on 8/1/11.
//  Copyright (c) 2009-2012 RestKit. All rights reserved.
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//  http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.
//

#import "VRKTableController.h"
#import <RestKit/RKAbstractTableController_Internals.h>
#import <RestKit/RKLog.h>
#import <RestKit/RKFormSection.h>
#import <RestKit/NSArray+RKAdditions.h>
#import <RestKit/RKObjectMappingOperation.h>


@implementation VRKTableController
@synthesize isRefresh;
@synthesize rootRecourcePath;

- (void)pullToRefreshStateChanged:(UIGestureRecognizer *)gesture {
    if (gesture.state == UIGestureRecognizerStateRecognized) {
        if ([self pullToRefreshDataSourceIsLoading:gesture])
            return;
        RKLogDebug(@"%@: pull to refresh triggered from gesture: %@", self, gesture);
        if (self.objectLoader) {
            [self.objectLoader reset];
            [self.objectLoader setResourcePath:rootRecourcePath];
            isRefresh = YES;
            [self.objectLoader send];
        }
    }
}

- (void)loadTableFromResourcePath:(NSString*)resourcePath {
    NSAssert(self.objectManager, @"Cannot perform a network load without an object manager");
    if (!self.rootRecourcePath)
        self.rootRecourcePath = resourcePath;
    isRefresh = YES;
    [self loadTableWithObjectLoader:[self.objectManager loaderWithResourcePath:resourcePath]];
}

-(void)loadMoreTableFromRecourcePath:(NSString*)resourcePath
{
    NSAssert(self.objectManager, @"Cannot perform a network load without an object manager");
    isRefresh = NO;
    [self loadTableWithObjectLoader:[self.objectManager loaderWithResourcePath:resourcePath]];
}

// NOTE - Everything currently needs to pass through this method to pick up header/footer rows...
- (void)loadObjects:(NSArray *)objects inSection:(NSUInteger)sectionIndex {
    // Clear any existing error state from the table
    self.error = nil;

    RKTableSection* section = [super sectionAtIndex:sectionIndex];
    
    int startPos = 0;
    if (isRefresh) {
        startPos = 0;
        section.objects = [super objectsWithHeaderAndFooters:objects forSection:sectionIndex];
    }
    else {
        startPos = [section.objects count];
        NSMutableArray* array = [NSMutableArray arrayWithArray:section.objects];
        [array addObjectsFromArray:[super objectsWithHeaderAndFooters:objects forSection:sectionIndex]];
        section.objects = array;
    }
    
    [self.tableView reloadData];
    
    if ([self.delegate respondsToSelector:@selector(tableController:didLoadObjects:inSection:)]) {
        [self.delegate tableController:self didLoadObjects:objects inSection:section];
    }
    
    // The load is finalized via network callbacks for
    // dynamic table controllers
    if (nil == self.objectLoader) {
        [self didFinishLoad];
    }
}


@end
